/*
	Tray Application to monitor battery status of rebuilt battery for Toshiba 50ct
	
	- OEM battery meter is inaccurate with 1800mah battery, this app recalibrates the battery level
	to be more accurate

	Author: Tavis Hord
	Date: 7/27/24
*/

#include "stdafx.h"
#include "resource.h"
#include <windows.h>
#include <shellapi.h>
#include <tchar.h>
#include <stdio.h>
#include <mmsystem.h>
#include <time.h>
#include <winuser.h> // Add this include at the top for ExitWindowsEx

// For sound
#pragma comment(lib, "winmm.lib")

#define MAX_LOADSTRING 100
#define TIMER_ID 1
#define BATTERY_UPDATE_TIMER_ID 2
#define TIMER_INTERVAL 1000 // 1 second
#define BATTERY_UPDATE_INTERVAL 5000 // 5 seconds

#define WINDOW_WIDTH 250
#define WINDOW_HEIGHT 185

// Timer length when battery status is 1%
#define BATTERY_CRITICAL_TIME 3600 // 1 hour in seconds

// Minimum allowed starting charge %
#define BATTERY_MIN_PERCENT 90 

// Global Variables:
HINSTANCE hInst; // current instance

TCHAR szTitle[MAX_LOADSTRING]; // app title bar text
TCHAR szWindowClass[MAX_LOADSTRING]; // window title bar text

//variables to store icons
HICON hIconFull, hIconHalf, hIconLow, hIconOff, hIconX;
NOTIFYICONDATA nid; // Data structure to hold tray icon information

#define WM_TRAYICON (WM_USER + 1) // Custom message for tray icon interaction
#define ID_TRAY_EXIT (WM_USER + 2) // Custom message for exit menu item
#define ID_TRAY_ABOUT (WM_USER + 3) // Custom message for exit menu item

bool timerStarted = false;
bool countdownStarted = false;
bool countdownAllowed = false;
time_t timerEndTime;
bool alertShown = false;
bool dontShowAgain = false;

bool acStatusPrevious = true; // Track previous AC line status (initialized to true assuming it starts connected)

int batteryPercent = 0;
int remainingTime = 0;

ATOM MyRegisterClass(HINSTANCE hInstance);
BOOL InitInstance(HINSTANCE, int);
LRESULT CALLBACK WndProc(HWND, UINT, WPARAM, LPARAM);
void ShowContextMenu(HWND hwnd, POINT pt);
void StartTimer();
void ResetTimer();
void GetBatteryStatus(HWND hwnd, TCHAR* szStatus, int bufferSize);
void LoadState();
void SaveState();
int GetBattPercent();
void CheckAndShowAlert(HINSTANCE hInst, HWND hWnd);

//:::: Dialog Boxes ::::

//About
LRESULT CALLBACK About(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
    switch (message)
    {
        case WM_INITDIALOG:
            return TRUE;

        case WM_COMMAND:
            if (LOWORD(wParam) == IDOK || LOWORD(wParam) == IDCANCEL) 
            {
                EndDialog(hDlg, LOWORD(wParam));
                return TRUE;
            }
            break;
    }
    return FALSE;
}

// Inactive Calibraiton DIalog
INT_PTR CALLBACK AlertDlgProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam) {
    switch (message) {
        case WM_INITDIALOG:
            return (INT_PTR)TRUE;

        case WM_COMMAND:
            if (LOWORD(wParam) == IDOK || LOWORD(wParam) == IDCANCEL) {
                // Check if the "Don't show this again" checkbox is checked
                dontShowAgain = (IsDlgButtonChecked(hDlg, IDC_CHECKBOX_DONTSHOWAGAIN) == BST_CHECKED);
                EndDialog(hDlg, LOWORD(wParam));
                return (INT_PTR)TRUE;
            }
            break;
    }
    return (INT_PTR)FALSE;
}

//show the alert dialog with "Don't show this again" option
void ShowAlertDialog(HINSTANCE hInst, HWND hWnd) {
    // Use DialogBox instead of DialogBoxParam
	INT_PTR result = DialogBox(hInst, (LPCTSTR)IDD_INACTIVE_ALERT, hWnd, (DLGPROC)AlertDlgProc);

    if (dontShowAgain) {
        SaveState();
    }
}

void CheckAndShowAlert(HINSTANCE hInst, HWND hWnd) {
    if (!dontShowAgain) {
        ShowAlertDialog(hInst, hWnd);
    }
}

void UpdateTaskbarIcon() {

	int batteryLevel = GetBattPercent();

	if (batteryLevel > 50) {
        nid.hIcon = hIconFull;
    } else if (batteryLevel > 20) {
        nid.hIcon = hIconHalf;
    } else {
        nid.hIcon = hIconLow;
    }

	if(batteryLevel == 0) {
		nid.hIcon = hIconOff;
	}

	if(batteryLevel == -1) {
		nid.hIcon = hIconX;
	}

    Shell_NotifyIcon(NIM_MODIFY, &nid);
}

int GetBattPercent() {
	SYSTEM_POWER_STATUS sps;
	
	GetSystemPowerStatus(&sps);
	
	int battPercent = sps.BatteryLifePercent;

	if(sps.ACLineStatus == 1) {
		return -1; //plugged in, no battery monitoring...
	}

	if(!countdownAllowed) { //calibration disabled... return OEM percent
		if(battPercent == 1){
			return 0;
		} else {
			return battPercent;
			//return 0; //lets not show batt level unless active calibration mode
		}
	} else {

		// Calculate the total percentage OEM % and calibrated timer.

		int rightBarPercent;
		int leftBarPercent;

		if(battPercent > 1) {
			rightBarPercent = battPercent / 2;
		} else {
			rightBarPercent = 0;
		}
		if (!timerStarted) {
			leftBarPercent = 50;
		} else {
			leftBarPercent = (remainingTime * 100 / BATTERY_CRITICAL_TIME) / 2;
		}

		int totalPercentage = leftBarPercent + rightBarPercent;
		
		return totalPercentage;
	}

}

int APIENTRY WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow) {
    MSG msg;

    // Load the saved state
    LoadState();

	// Load icons
    hIconFull = LoadIcon(GetModuleHandle(NULL), MAKEINTRESOURCE(IDI_BATT_100));
    hIconHalf = LoadIcon(GetModuleHandle(NULL), MAKEINTRESOURCE(IDI_BATT_50));
    hIconLow = LoadIcon(GetModuleHandle(NULL), MAKEINTRESOURCE(IDI_BATT_20));
    hIconOff = LoadIcon(GetModuleHandle(NULL), MAKEINTRESOURCE(IDI_BATTMON));
	hIconX = LoadIcon(GetModuleHandle(NULL), MAKEINTRESOURCE(IDI_BATT_X));


    // Initialize global strings
    LoadString(hInstance, IDS_APP_TITLE, szTitle, MAX_LOADSTRING);
    LoadString(hInstance, IDC_BATTMON, szWindowClass, MAX_LOADSTRING);

    // Register window class
    if (!MyRegisterClass(hInstance)) {
        MessageBox(NULL, _T("Call to RegisterClassEx failed!"), _T("Windows Desktop Guided Tour"), NULL);
        return 1;
    }

    // Perform application initialization
    if (!InitInstance(hInstance, SW_HIDE)) { // Start hidden
        MessageBox(NULL, _T("Call to InitInstance failed!"), _T("Windows Desktop Guided Tour"), NULL);
        return 1;
    }

    // Initialize NOTIFYICONDATA structure
    ZeroMemory(&nid, sizeof(nid));
    nid.cbSize = sizeof(nid);
    nid.hWnd = FindWindow(szWindowClass, szTitle);
    nid.uID = 1;
    nid.uFlags = NIF_ICON | NIF_MESSAGE | NIF_TIP;
    nid.uCallbackMessage = WM_TRAYICON;
    nid.hIcon = LoadIcon(hInstance, MAKEINTRESOURCE(IDI_BATTMON));
    lstrcpy(nid.szTip, _T("BattMon 1.0.1"));

    // Add the icon to the system tray
    Shell_NotifyIcon(NIM_ADD, &nid);

    // Main message loop
    while (GetMessage(&msg, NULL, 0, 0)) {
        TranslateMessage(&msg);
        DispatchMessage(&msg);
    }

    return (int)msg.wParam;
}

void SaveState() {
    FILE* file = _tfopen(_T("battMonState.dat"), _T("wb"));
    if (file) {
        fwrite(&timerStarted, sizeof(timerStarted), 1, file);
        fwrite(&timerEndTime, sizeof(timerEndTime), 1, file);
        fwrite(&countdownAllowed, sizeof(countdownAllowed), 1, file); //has battery been fully charged before unplugging
		fwrite(&dontShowAgain, sizeof(dontShowAgain), 1, file); // Save the don't show again flag
        fclose(file);
    } else {
		//MessageBox(NULL, "cant save state", _T("Error"), MB_OK | MB_ICONERROR);
	}
}

void LoadState() {
    FILE* file = _tfopen(_T("battMonState.dat"), _T("rb"));
    if (file) {
        fread(&timerStarted, sizeof(timerStarted), 1, file);
        fread(&timerEndTime, sizeof(timerEndTime), 1, file);
		fread(&countdownAllowed, sizeof(countdownAllowed), 1, file); //has battery been fully charged before unplugging
		fread(&dontShowAgain, sizeof(dontShowAgain), 1, file); // Load the don't show again flag
        fclose(file);
    } else {
        // Set default values if the file does not exist
        timerStarted = false;
        alertShown = false;
    }
}

// Register the window class
ATOM MyRegisterClass(HINSTANCE hInstance) {
    WNDCLASSEX wcex;

    wcex.cbSize = sizeof(WNDCLASSEX);
    wcex.style = CS_HREDRAW | CS_VREDRAW;
    wcex.lpfnWndProc = WndProc;
    wcex.cbClsExtra = 0;
    wcex.cbWndExtra = 0;
    wcex.hInstance = hInstance;
    wcex.hIcon = LoadIcon(hInstance, MAKEINTRESOURCE(IDI_BATTMON));
    wcex.hCursor = LoadCursor(NULL, IDC_ARROW);
    wcex.hbrBackground = (HBRUSH)(COLOR_WINDOW + 1);
    wcex.lpszMenuName = NULL;
    wcex.lpszClassName = szWindowClass;
    wcex.hIconSm = LoadIcon(wcex.hInstance, MAKEINTRESOURCE(IDI_SMALL));

    return RegisterClassEx(&wcex);
}

// Save instance handle and create main window
BOOL InitInstance(HINSTANCE hInstance, int nCmdShow) {
    HWND hWnd;

    hInst = hInstance; // Store instance handle in our global variable

    // Calculate the window size to account for borders and title bar
    RECT rect = { 0, 0, WINDOW_WIDTH, WINDOW_HEIGHT };
    AdjustWindowRect(&rect, WS_OVERLAPPED | WS_CAPTION | WS_SYSMENU, FALSE);

    // Create window with specific style to remove minimize and maximize buttons
    hWnd = CreateWindow(szWindowClass, szTitle, WS_OVERLAPPED | WS_CAPTION | WS_SYSMENU, CW_USEDEFAULT, 0, rect.right - rect.left, rect.bottom - rect.top, NULL, NULL, hInstance, NULL);

    if (!hWnd) {
        DWORD dwError = GetLastError();
        TCHAR szError[256];
        _stprintf(szError, _T("CreateWindow failed with error %d"), dwError);
        MessageBox(NULL, szError, _T("Error"), MB_OK | MB_ICONERROR);
        return FALSE;
    }

    ShowWindow(hWnd, nCmdShow);
    UpdateWindow(hWnd);

    // Start the battery update timer
    SetTimer(hWnd, BATTERY_UPDATE_TIMER_ID, BATTERY_UPDATE_INTERVAL, NULL);

    return TRUE;
}

// Show context menu
void ShowContextMenu(HWND hwnd, POINT pt) {
    HMENU hMenu = CreatePopupMenu();
    if (hMenu) {
        // Add "About" menu item
        InsertMenu(hMenu, -1, MF_BYPOSITION, ID_TRAY_ABOUT, _T("About"));
        
        // Add "Exit" menu item
        InsertMenu(hMenu, -1, MF_BYPOSITION, ID_TRAY_EXIT, _T("Exit"));
        
        SetForegroundWindow(hwnd);
        TrackPopupMenu(hMenu, TPM_BOTTOMALIGN | TPM_RIGHTBUTTON, pt.x, pt.y, 0, hwnd, NULL);
        DestroyMenu(hMenu);
    }
}


void StartTimer() {
    timerStarted = true;
    //timerEndTime = time(NULL) + BATTERY_CRITICAL_TIME;
	timerEndTime = BATTERY_CRITICAL_TIME;
	SaveState();
}

void ResetTimer() {
    timerStarted = false;
    countdownStarted = false;
	countdownAllowed = false;
	alertShown = false;
    SaveState();
}

LRESULT CALLBACK WndProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam) {
		static TCHAR szBatteryStatus[256];
		static HBRUSH battBarBrush = CreateSolidBrush(RGB(100, 100, 255)); // Define brushes
		static HBRUSH timeBarBrush = CreateSolidBrush(RGB(140, 140, 255));
		//static HBRUSH whiteBrush = (HBRUSH)GetStockObject(WHITE_BRUSH);
   		static HBRUSH disabledBrush = CreateSolidBrush(RGB(169, 169, 169)); // Gray color for disabled state

		// progress bar background color and border color
		static HBRUSH backgroundBrush = CreateSolidBrush(RGB(220, 220, 220)); // Light grey background
		static HBRUSH borderBrush = CreateSolidBrush(RGB(0, 0, 0)); // Black border

		switch (msg) {

			case WM_COMMAND:
				// if (LOWORD(wParam) == ID_TRAY_EXIT) {
				// 	DestroyWindow(hwnd);
				// }
				switch (LOWORD(wParam)) {
					case ID_TRAY_ABOUT:
						// Show About dialog
						DialogBox(hInst, (LPCTSTR)IDD_ABOUTBOX, hwnd, (DLGPROC)About);
						break;
					case ID_TRAY_EXIT:
						DestroyWindow(hwnd);
						break;
				}
				break;

			case WM_PAINT: {
				PAINTSTRUCT ps;
				HDC hdc = BeginPaint(hwnd, &ps);
				RECT rt;
				GetClientRect(hwnd, &rt);

				// Calculate the total percentage
				int rightBarPercent;
				int leftBarPercent;

				if(batteryPercent > 1) {
					rightBarPercent = batteryPercent / 2;
				} else {
					rightBarPercent = 0;
				}
				if (!timerStarted) {
					leftBarPercent = 50;
				} else { 
					leftBarPercent = (remainingTime * 100 / BATTERY_CRITICAL_TIME) / 2;
				}
			    int totalPercentage = leftBarPercent + rightBarPercent;

				// Define the dimensions of the progress bar
				int progressBarWidth = rt.right - rt.left - 20; // Total width of the progress bar
				int progressBarHeight = 10; // Increased height for better visibility
				int barStartX = 10; // Starting X position of the bar
				int barStartY = 13; // Starting Y position of the bar

				// Calculate the width of the battery and time bars
				int halfProgressBarWidth = progressBarWidth / 2;
				int batteryBarWidth = halfProgressBarWidth * batteryPercent / 100;

				// Ensure the time bar is fully filled initially
				int timeBarWidth;
				if (!timerStarted) {
					timeBarWidth = halfProgressBarWidth; // Fully fill the time bar initially
				} else {
					timeBarWidth = halfProgressBarWidth * remainingTime / BATTERY_CRITICAL_TIME;
				}

				// Draw border
				HPEN hPenOld;
				HPEN hLinePen = CreatePen(PS_SOLID, 1, RGB(0, 0, 0)); // Black border
				hPenOld = (HPEN)SelectObject(hdc, hLinePen);
				Rectangle(hdc, barStartX-1, barStartY-1, barStartX + progressBarWidth +1, barStartY + progressBarHeight +1);
				SelectObject(hdc, hPenOld);
				DeleteObject(hLinePen);

				// Draw background
				RECT fullBarRect = { barStartX, barStartY, barStartX + progressBarWidth, barStartY + progressBarHeight };
				FillRect(hdc, &fullBarRect, backgroundBrush); 

				//::: Draw Progress bar :::

				// Draw time remaining bar (left half)
				RECT timeRect = { barStartX, barStartY, barStartX + timeBarWidth, barStartY + progressBarHeight };
				FillRect(hdc, &timeRect, countdownAllowed ? timeBarBrush : disabledBrush); // Use gray brush if countdown is not allowed
				
				// Draw battery percentage bar (right half)
				if(batteryBarWidth > 1) { //prevent single pixel vertical line and only draw if theres a with
					RECT batteryRect = { barStartX + halfProgressBarWidth, barStartY, barStartX + halfProgressBarWidth + batteryBarWidth, barStartY + progressBarHeight };
					FillRect(hdc, &batteryRect, battBarBrush);
				}
				
				// ::: Draw total percentage text :::
				
				// Calculate the estimated remaining time based on the battery percentage
				int estimatedRemainingTimeFromBattery;
				int batteryRuntime = BATTERY_CRITICAL_TIME;
				if (batteryPercent == 100) {
					estimatedRemainingTimeFromBattery = batteryRuntime;
				} else {
					// Avoid floating point arithmetic and division by zero
					estimatedRemainingTimeFromBattery = (batteryRuntime * batteryPercent) / 100;
				}

				int totalRemainingTime;
				int totalHours;
				int totalMinutes;

				if (!remainingTime) {
					totalRemainingTime = estimatedRemainingTimeFromBattery + BATTERY_CRITICAL_TIME;
				} else {
					totalRemainingTime = remainingTime;
				}
			
				totalHours = totalRemainingTime / 3600;
				totalMinutes = (totalRemainingTime % 3600) / 60;

				// Draw total percentage and remaining time text
				TCHAR totalPercentageText[100];
				_stprintf(totalPercentageText, _T("Calibrated Battery: %d%%\nTime Remaining: %02d:%02d"), countdownAllowed ? totalPercentage : 0, countdownAllowed ? totalHours : 0, countdownAllowed ? totalMinutes : 0);
				RECT textRect = { barStartX, barStartY + progressBarHeight + 15, barStartX + progressBarWidth, barStartY + progressBarHeight + 50 };
				SetBkMode(hdc, TRANSPARENT);
				DrawText(hdc, totalPercentageText, -1, &textRect, DT_LEFT | DT_VCENTER);

				// Draw a horizontal separator line
				COLORREF grayColor = RGB(220, 220, 220); // Medium gray
				HPEN hGrayPen = CreatePen(PS_SOLID, 1, grayColor);
				HPEN hOldPen = (HPEN)SelectObject(hdc, hGrayPen);

				MoveToEx(hdc, barStartX, textRect.bottom + 7, NULL);
				LineTo(hdc, barStartX + progressBarWidth, textRect.bottom + 7);

				SelectObject(hdc, hOldPen);
				DeleteObject(hGrayPen);

				// Draw status text
				rt.top += textRect.bottom;
				rt.left = textRect.left;

				SetBkMode(hdc, TRANSPARENT);
				DrawText(hdc, szBatteryStatus, -1, &rt, DT_WORDBREAK | DT_LEFT | DT_VCENTER);

				// Cleanup
				//DeleteObject(backgroundBrush);
				//DeleteObject(borderBrush);

				EndPaint(hwnd, &ps);
				break;
			}

        case WM_TIMER:
            if (wParam == BATTERY_UPDATE_TIMER_ID) {
                GetBatteryStatus(hwnd, szBatteryStatus, sizeof(szBatteryStatus) / sizeof(TCHAR));
                InvalidateRect(hwnd, NULL, TRUE);
			}
            break;

        case WM_TRAYICON:
            if (lParam == WM_LBUTTONDOWN) {
                if (IsWindowVisible(hwnd)) {
                    ShowWindow(hwnd, SW_HIDE);
                } else {
                    ShowWindow(hwnd, SW_SHOW);
                    SetForegroundWindow(hwnd);
                }
            } else if (lParam == WM_RBUTTONDOWN) {
                POINT pt;
                GetCursorPos(&pt);
                ShowContextMenu(hwnd, pt);
            }
            return 0;

        case WM_CLOSE:
            ShowWindow(hwnd, SW_HIDE);
            return 0;

        case WM_DESTROY:
            KillTimer(hwnd, TIMER_ID);
            KillTimer(hwnd, BATTERY_UPDATE_TIMER_ID);
            Shell_NotifyIcon(NIM_DELETE, &nid);
            DeleteObject(battBarBrush);
            DeleteObject(timeBarBrush);
            DeleteObject(disabledBrush);
            DeleteObject(backgroundBrush);
            DeleteObject(borderBrush);			
            PostQuitMessage(0);
            break;

        case WM_GETMINMAXINFO: {
            MINMAXINFO* pMinMaxInfo = (MINMAXINFO*)lParam;
            if (pMinMaxInfo) {
                pMinMaxInfo->ptMinTrackSize.x = WINDOW_WIDTH;
                pMinMaxInfo->ptMinTrackSize.y = WINDOW_HEIGHT;
                pMinMaxInfo->ptMaxTrackSize.x = WINDOW_WIDTH;
                pMinMaxInfo->ptMaxTrackSize.y = WINDOW_HEIGHT;
            }
            return 0;
        }
        default:
            return DefWindowProc(hwnd, msg, wParam, lParam);
    }
    return 0;
}

void GetBatteryStatus(HWND hwnd, TCHAR* szStatus, int bufferSize) {
    SYSTEM_POWER_STATUS sps;

    if (GetSystemPowerStatus(&sps)) {
        // Update battery percent
        batteryPercent = sps.BatteryLifePercent;

        static bool alerted = false;

		//handle condition when offline full charge occured while unplugged prior to sleep
		if(sps.BatteryLifePercent >= BATTERY_MIN_PERCENT && sps.ACLineStatus == 0 && acStatusPrevious == 0) {
			//battery is fully charged while unplugged state
			//we may have been hibernating while charging
			//and unplgged before waking up...

			// acStatusPrevious is false now so we cant kick off the timer
			// force the switch to occur

			if(!countdownAllowed) {
				acStatusPrevious = 1;
			}
		} else {
			//handle condition when offline partial offline charging during active discharge
			if(sps.BatteryLifePercent > 1 && sps.BatteryLifePercent < BATTERY_MIN_PERCENT && sps.ACLineStatus == 0 && acStatusPrevious == 0) {
				//battery is > 1% 
				//battery is < full		
				if(countdownAllowed && timerStarted) { //calib mode active and in phase 2 discharge
					ResetTimer(); //force inactive (partial charging occured offline)
					acStatusPrevious = 1; //force online / offline switch
				}
			}
		}

		if (acStatusPrevious && sps.ACLineStatus == 0) { //switched from a/c to battery
            if (!countdownAllowed) { // Check if we are already running
                if (sps.BatteryLifePercent <= BATTERY_MIN_PERCENT) { //OEM batt level < fully charged
                    if (!timerStarted && !alerted) {
                        alerted = true;
						CheckAndShowAlert(hInst, hwnd);
						//MessageBox(NULL, _T("You must start with a fully charged battery for active calibration.\n\nInactive calibration results in inaccurate system status and you will not be notified when battery is low."), _T("BattMon Alert"), MB_OK | MB_ICONWARNING);
						ResetTimer();
                    }
                } else { //batt is fully charged, start calibration mode
                    countdownAllowed = true;
					SaveState();
				}
            }
        } else if (!acStatusPrevious && sps.ACLineStatus == 1) { //switched from battery to a/c
			if(countdownAllowed && sps.BatteryLifePercent > 1) {
				//resume as normal still > 1% and timer hasnt started yet
			} else {
				ResetTimer();
				alerted = false;
        }	}

		//save current state
        acStatusPrevious = (sps.ACLineStatus == 1); //ACLineStatus 1 = external a/c

		//calibraiton mode is enabled and we are unplugged
		//start timer when OEM status is 1%
        if (countdownAllowed && sps.ACLineStatus == 0) { //ACLineStatus 0 = battery

			
			

            if (sps.BatteryLifePercent <= 1) { //start the timer when OEM batt gets to 1%
                if (!timerStarted) {
                    StartTimer();
                }

                //remainingTime = (int)difftime(timerEndTime, time(NULL)); //get remaining time
				timerEndTime -= (BATTERY_UPDATE_INTERVAL / 1000);
				remainingTime = timerEndTime;
				SaveState(); //save remaining time incase we shutdown

				if (remainingTime <= 60) {
					remainingTime = 60; //hold at 1 minute remaining
					ResetTimer(); // Call the suspend function
                }

                if (remainingTime <= 600 && !alertShown) {
                    alertShown = true;
                    PlaySound(TEXT("SystemHand"), NULL, SND_ALIAS | SND_ASYNC);
                    MessageBox(NULL, _T("Battery is getting low. Plug in soon."), _T("Battery Alert"), MB_OK | MB_ICONWARNING);
                }
            } else {
                remainingTime = 0; 
            }
        } else {
            remainingTime = 0;
        }

        if (sps.ACLineStatus == 1) {

			_stprintf(szStatus, _T("\nPower Source: %s\nCalibration: Inactive\nSystem Battery: %d%%"),
			(sps.ACLineStatus == 1) ? _T("External") : _T("Battery"),
			sps.BatteryLifePercent);
        } else {
            int hours = remainingTime / 3600;
            int minutes = (remainingTime % 3600) / 60;

            if (countdownAllowed) {

				_stprintf(szStatus, _T("\nPower Source: %s\nCalibration: Active\nSystem Battery: %d%%"),
                        (sps.ACLineStatus == 1) ? _T("External") : _T("Battery"),
                        sps.BatteryLifePercent);		
            } else {

				_stprintf(szStatus, _T("\nPower Source: %s\nCalibration: Inactive\nSystem Battery: %d%%"),
                        (sps.ACLineStatus == 1) ? _T("External") : _T("Battery"),
                        sps.BatteryLifePercent);

            }
        }

    	UpdateTaskbarIcon();

    } else {
        _tcscpy(szStatus, _T("Unable to retrieve System Battery Status."));
    }
}


